#! /bin/sh

##########
## Base ##
##########

ignore_errors=0
failing_test=0

stderr ()
{
  local i
  local input_local
  test x"$input" != x && input_local="$input: "
  for i
  do
    echo >&2 "$0: $base_local$i"
  done
}

fatal ()
{
  stderr "$@"
  test $ignore_errors -ne 0 || exit 1
  return_code=1
}

warn ()
{
  stderr "WARNING: $@"
}

assert_file ()
{
  local file
  eval file="\$$1"
  test -f "$file" ||
    fatal "no such file: $1 = $file"
}


##############
## Programs ##
##############

path_progs()
{
# looking for the good path separator
  if test "${PATH_SEPARATOR+set}" != set; then
    PATH_SEPARATOR=:
      (PATH='/bin;/bin'; FPATH=$PATH; sh -c :) >/dev/null 2>&1 && {
        (PATH='/bin:/bin'; FPATH=$PATH; sh -c :) >/dev/null 2>&1 ||
          PATH_SEPARATOR=';'
      }
  fi

# then try to find the executable in $PATH
  as_save_IFS=$IFS; IFS=$PATH_SEPARATOR
  for as_dir in $PATH
  do
    IFS=$as_save_IFS
    test -z "$as_dir" && as_dir=.
    for as_exec_ext in '' $ac_executable_extensions; do
      if test -x "$as_dir/$1$ac_exec_ext"; then
        found="$as_dir/$1$ac_exec_ext"
        break 3
      fi
    done
  done
  IFS=$as_save_IFS
  if ! test -n "$found"; then
    fatal "cannot find $1"
  fi
}

run()
{
  path_progs "$1"
  echo "$0: running $@"
    "$@" || fatal "$1 failed with exit status: $?"
}


###########
## TESTS ##
###########


run_test ()
{
  local old_return_code=$return_code
  local old_ignore_errors=$ignore_errors; ignore_errors=1
  "$@"
  test $failing_test -eq 0 ||
    return_code=$((! $return_code))
  ignore_errors=$old_ignore_errors;
  return_code=$(($old_return_code || $return_code))
  return 0
}

# compilation SOURCE DEST
compilation ()
{
  local in=$1
  local out=$2

  if test "x$TEST_COMPILER" = x; then
    warn "no \$TEST_COMPILER set, falling back to g++"
    TEST_COMPILER=g++
  fi
  path_progs "$TEST_COMPILER"
  $TEST_COMPILER -x c++ -c $in -o $out
  exit_status=$?
  test -e $out && rm -f $out
  exit $exit_status
}

# parse PARSER SOURCE DEST PARSER_ARGS
parse ()
{
    local parser=$1
    local in=$2
    local out=$3
    shift 3
    failing_test=0
    if $(grep "^//FAIL:" "$dir/$in" &> /dev/null); then
      failing_test=1
    fi

    assert_file parser
    assert_file BUILD_REPOSITORY

    : ${VERBOSE=1}
    : ${KEEP=0}

    oldPWD="$PWD"
    cd $dir

    set +e
    XTC_REPOSITORY=$BUILD_REPOSITORY	\
         "$parser"	        		\
            --verbose $VERBOSE			\
            --keep $KEEP			\
            "$@"				\
            -i "$in"				\
            -o "$out"
    res=$?
    set -e
    test $res -ge 128 && exit 177 # terminated by a signal, hard fail

    if test $res -ne 0; then
      test $failing_test -eq 0 &&
        fatal "$parser failed"
    else
      test $failing_test -eq 0 ||
        fatal "$parser pass, but it shouldn't"
    fi
    cd "$oldPWD"
}

# parse_pipeline PARSER SOURCE PARSER_ARGS
parse_pipeline ()
{
    local parser=$1
    local in=$2
    shift 2
    failing_test=0
    if $(grep "^//FAIL:" "$dir/$in" &> /dev/null); then
      failing_test=1
    fi

    assert_file parser
    assert_file BUILD_REPOSITORY

    : ${VERBOSE=1}
    : ${KEEP=0}

    oldPWD="$PWD"
    cd $dir
    if ! XTC_REPOSITORY=$BUILD_REPOSITORY	\
         "$parser"	        		\
            --verbose $VERBOSE			\
            --keep $KEEP			\
            "$@"				\
            -i "$in"; then
      test $failing_test -eq 0 &&
        fatal "$parser failed"
    else
      test $failing_test -eq 0 ||
        fatal "$parser pass, but it shouldn't"
    fi
    cd "$oldPWD"
}

# _parse_cxx SOURCE DEST PARSE_CXX_ARGS
parse_cxx ()
{
    parse_cxx=$abs_top_builddir/cxx-grammar/src/parser/parse-cxx
    parse $parse_cxx "$@"
}

# parse_dxx SOURCE PARSE_CXX_ARGS
parse_dxx ()
{
    parse_cxx "$@" -dxx strict
}


# _parse_c SOURCE DEST PARSE_C_ARGS
parse_c ()
{
    parse_c=$abs_top_builddir/c-grammar/src/parser/parse-c99
    parse $parse_c "$@"
}

# parse_gnuc SOURCE DEST PARSE_C_ARGS
parse_gnuc ()
{
    parse_c "$@" -p GnuC
}

# parse_d SOURCE DEST PARSE_C_ARGS
parse_d ()
{
    parse_c "$@" -d strict
}



test_diff_detc ()
{
    local return_code=0
    run_test parse_c "$input" "output.c" "$@"
    run_test parse_d "$input" "output.detc" "$@"
    if test $failing_test -eq 1; then
      return_code=$((! $return_code))
      exit $return_code
    fi
    test $return_code -eq 0 || exit $return_code
    diff >&2 "$dir/output.c" "$dir/output.detc" >"$dir/output.diff"
    if test -s "$dir/output.diff"; then
      stderr "Test failed"
      cat >&2 "$dir/output.diff"
      exit 1
    fi
}
test_diff_detcc ()
{
    local return_code=0
    run_test parse_cxx "$input" "output.cxx" "$@"
    run_test parse_dxx "$input" "output.dxx" "$@"
    if test $failing_test -eq 1; then
      return_code=$((! $return_code))
      echo global res $return_code
      exit $return_code
    fi
    test $return_code -eq 0 || exit $return_code
    echo diff
    diff >&2 "$dir/output.cxx" "$dir/output.dxx" >"$dir/output.diff"
    if test -s "$dir/output.diff"; then
      cat >&2 "$dir/output.diff"
      fatal "Test failed"
    fi
}

timeout_set ()
{
   sleep $2
   kill -s ALRM $1
}

test_main ()
{
    set -e
    ulimit -m 524288 -t 10
    timeout_set $$ 60 &

    test $# -ne 1 && fatal "Usage: $0 input [args]"
    input=$1
    shift
    test -f "$input" ||
        fatal "no such file: $input"
    ext=`echo "$input" | sed 's/.*\.//g'`
    dir=`echo "$logfile" | sed 's/-parse\.log$//;s/-compile\.log$//;s/\.log$//'`.dir
    input=`echo "$dir" | sed 's/[^/]*/../g'`/"$input"
    noext=`echo "$input" | sed 's/\.[^.]*$//'`
    rm -rf "$dir"
    mkdir "$dir"
    _dispatch "$@"
    exit 0
}
